;;; mouse commands
        .equ MOUSE_RESET_CMD         , 0xFF 
        .equ MOUSE_ENABLE_REPORT_CMD , 0xF4
        .equ MOUSE_SET_RESOLUTION_CMD, 0xE8
        .equ MOUSE_SET_RESOLUTION_VAL_1PERMM, 0x00
        .equ MOUSE_SET_RESOLUTION_VAL_4PERMM, 0x02


;;; I/O Ports and Masks

        .equ PS2_DAT_IN      , PSDAT_I
        .equ PS2_DAT_IN_MASK , 1

        .equ PS2_CLK_IN      , PSCLK_I
        .equ PS2_CLK_IN_MASK , 1

        .equ PS2_DAT_OUT     , PSDAT_O
        .equ PS2_DAT_OUT_MASK, 1

        .equ PS2_CLK_OUT     , PSCLK_O
        .equ PS2_CLK_OUT_MASK, 1        
        

        .equ PS2_CLK_FF_CLR   , CL_PSCLK

        
        
        .equ TEN_USEC_CONST, 35 ;TEN_USEC_CONST = (10*f_MHz - 7 - 8 ) / 13 + 1


;;; mouse_init: mouse init helper
;;; 
;;; Sequence:
;;; - Init PS2 comm
;;; - Send 'Reset' cmd
;;; - Receive 'Reset' ack       
;;; - Receive 'BAT Status' response    
;;; - Receive 'device id'
;;; - Send 'Enable reporting' cmd
;;; - Receive 'Enable reporting' ack
;;; 
mouse_init:
    ;; Put clk and dat lines idle
        call ps2_init
    ;; Send 'Reset' cmd and receive its ack
reset_cmd:     
        ld a, MOUSE_RESET_CMD
        call intup_send
reset_ack:
	    call intup_sub2
reset_ack_response:
        nop                     ; A should be 0xFA
    
    ;; Receive 'BAT status' after reset
bat_status:
        call intup_sub2
bat_status_response:
        nop                     ; A should be 0xAA
    
        ;; Wait for the mouse to send its device id (0x00)
wait_for_dev_id:
        call intup_sub2
dev_id_response:
        cp 0
        jr nz, wait_for_dev_id

        ;; Send 'Enable reporting' cmd and receive its ack
enable_report_cmd:      
		call wait_100us
		call wait_100us
        ld a, MOUSE_ENABLE_REPORT_CMD
        call intup_send
enable_report_ack:    
        call intup_sub2       ; receive ack
enable_report_ack_response:     ; A should be 0xFA
        cp 0xFA					; some mouses send response 0xFC (error)
		jr nz, enable_report_cmd
		
mouse_init_end:
		call wait_100us			; some mouses have ck activity for some time after last respose ..?
		call wait_100us
		out (PS2_CLK_FF_CLR), a
        ret
;;; end of mouse_init

ps2_init:
        push af
        ld a, 0xFF
        out (PS2_DAT_OUT), a    ; make sure DAT and CLK lines are released
        out (PS2_CLK_OUT), a    ; make sure DAT and CLK lines are released
        pop af
        ret

;;; intup_send:
;;; send the byte in A through PS2
intup_send:
        ld c, a
        call ps2_rts            ; request to send
        call intup_sub1

        ld b, 8
serialization_loop:
        ;; send the next bit
        ld a, PS2_DAT_OUT_MASK
        cpl                     ; all ones except for a zero in the DAT bit

        rrc c                   ; 8-bit rotation to the right, next bit to send goes to the carry
        jr nc, ps2_port_write
        or PS2_DAT_OUT_MASK

ps2_port_write:
        out (PS2_DAT_OUT), a

        call intup_sub1
        djnz serialization_loop ; serialize the next bit

calc_parity:
        ld a, PS2_DAT_OUT_MASK
        cpl                     ; all ones except for a zero in the DAT bit
        rrc c                   ; just an easy way to get the parity, any other op will do
        jp po, parity_bit
        or PS2_DAT_OUT_MASK     ; if parity is odd, we need to send a '1'

parity_bit:
        out (PS2_DAT_OUT), a
        call intup_sub1

stop_bit:
        ;; release DAT line
        ld a, 0xff
        out (PS2_DAT_OUT), a

;;;     call wait_dat_fall    ; wait for the device to bring data low
        out (PS2_CLK_FF_CLR), a
        call intup_sub1

        call wait_dev_idle      ; wait for the device to release both DAT and CLK lines

intup_send_end:
        ret


;;; intup_sub2:
intup_sub2:
        .db 0xc5
        .db 0xdb
        .db PS2_CLK_IN
        .db 0xe6
        .db 0x01
        .db 0x28
        .db 0xfa
        .db 0xd3
        .db PS2_CLK_FF_CLR
        .db 0xdb
        .db PS2_DAT_IN
        .db 0xe6
        .db 0x01
        .db 0x20
        .db 0xf2
        .db 0x06
        .db 0x08
        .db 0x0e
        .db 0x00
        .db 0xdb
        .db PS2_CLK_IN
        .db 0xe6
        .db 0x01
        .db 0x28
        .db 0xfa
        .db 0xd3
        .db PS2_CLK_FF_CLR
        .db 0xdb
        .db PS2_DAT_IN
        .db 0xe6
        .db 0x01
        .db 0x1f
        .db 0xcb
        .db 0x19
        .db 0x10
        .db 0xef
        .db 0xdb
        .db PS2_CLK_IN
        .db 0xe6
        .db 0x01
        .db 0x28
        .db 0xfa
        .db 0xd3
        .db PS2_CLK_FF_CLR
        .db 0xdb
        .db PS2_DAT_IN
        .db 0xe6
        .db 0x01
        .db 0x1f
        .db 0x79
        .db 0xc1
intup_sub2_end:
        .db 0xc9

;;; ps2_rts: implement PS2 "request to send" sequence.
;;;  - bring CLK line low
;;;  - wait 100us
;;;  - bring DAT line low
;;;  - release CLK line
ps2_rts:
        ;; save registers
        push af
        push bc

        ;; bring CLK low
        ld a, PS2_CLK_OUT_MASK
        cpl
        out (PS2_CLK_OUT), a
        out (PS2_CLK_FF_CLR), a ; TODO: is this necessary? maybe move it somewhere else (ie, before intup_sub2)

        ;; wait 100 usec
        call wait_100us

        ;; bring DAT low and release CLK
        ld a, PS2_DAT_OUT_MASK  ; 0001
        cpl                     ; 1110
        out (PS2_DAT_OUT), a    ; clk=1 dat=0
        ld a, PS2_CLK_OUT_MASK  ; 0010
        out (PS2_CLK_OUT), a    ; clk=1 dat=0

        ;; restore registers
        pop bc
        pop af
        out (PS2_CLK_FF_CLR), a ; TODO: is this necessary? maybe move it somewhere else (ie, before intup_sub2)
ps2_rts_end:
        ret

;;; wait 100 usecs by waiting 10 times for 10 usecs
wait_100us:
        push af
        push bc

        ld b, 10
wait_loop_100:
        call wait_10us
        djnz wait_loop_100

wait_100us_end:
        pop bc
        pop af
        ret

;;; wait 10 usecs routine
wait_10us:
        push af
        push bc

        ld b, TEN_USEC_CONST
wait_loop_10:
        djnz wait_loop_10

wait_10us_end:
        pop bc
        pop af
        ret
    
;; intup_sub1
intup_sub1:
        in a, (PS2_CLK_IN)
        and PS2_CLK_IN_MASK
        jr z, intup_sub1
        out (PS2_CLK_FF_CLR), a ; clear falling edge detection flipflop
        ret

;; wait for rising PS2 DAT
wait_dat_rise:
        in a, (PS2_DAT_IN)
        and PS2_DAT_IN_MASK
        jr z, wait_dat_rise
        ret

wait_dev_idle:
        call wait_dat_rise
        ret
